#include <stdlib.h>
#include <time.h>
#include <string.h>
#include <allegro.h>

#define GLOBALS         /* To declare _somewhere_ all that stuff... */
#include "defines.h"

#include "cmndline.h"
#include "game.h"
#include "main.h"

void     ticker (void)
{
   tick++;
   delay[0]++;
   delay[1]++;
   reload[0]++;
   reload[1]++;
}
END_OF_FUNCTION(ticker);

void     recharger (void)
{
   recharge++;
}
END_OF_FUNCTION(recharger);


/**[txh]********************************************************************

  Description: Main loop? Just inits stuff, process commandline and gives
  power to the real game.

  Return: (int) Zero on success, other value on failure.

***************************************************************************/

int      main (int argc, char *argv[])
{
   srand (time (NULL));
   allegro_init();
   cpu_mmx = FALSE;
   
   if (parse_command_line (argc, argv))
      return 2;

   if (enter_graphic_mode())
      return 3;

   if (post_init())
      return 4;

   while (!GET (flag, QUIT_GAME))
      run_game();

   return shutdown_game();
}
END_OF_MAIN();


/**[txh]********************************************************************

  Description: Assuming the state of a full opened game, this closes
  everything.

  Return: (int) zero.

***************************************************************************/

int      shutdown_game (void)
{
#define  DB    destroy_bitmap
   
   DB (back);
   DB (ship);
   DB (sub);
   DB (mine);
   DB (load);
   DB (table);

   remove_keyboard();
   remove_timer();
   set_gfx_mode (GFX_TEXT, 0, 0, 0, 0);
   allegro_exit();

   return 0;
}


/**[txh]********************************************************************

  Description: Not only enters the graphic mode but also initializes other
  stuff like keyboard or timers.

  Return: (int) Nonzero if game can't continue.

***************************************************************************/

int      enter_graphic_mode (void)
{
   PALETTE pal;

   if (set_gfx_mode (GFX_AUTODETECT, w, h, 0, 0))
   {
      allegro_message ("%s %dx%d: %s\n", get_config_text ("error_setting"),
         w, h, allegro_error);
      return 1;
   }

   generate_332_palette (pal);
   set_palette (pal);
   clear_to_color (screen, BLACK);
   text_mode (-1);

   install_timer();
   install_keyboard();

   #define LV LOCK_VARIABLE
   #define LF LOCK_FUNCTION

   LV (tick);
   LV (recharge);
   LF (ticker);
   LF (recharger);
   install_int_ex (ticker, BPS_TO_TIMER (TICKS));
   install_int_ex (recharger, SECS_TO_TIMER (3));

   return 0;
}

/**[txh]********************************************************************

  Description: Post initialization. In short, creates all the bitmaps and
  sets useful variables... at least we are in graphic mode...

  Return: (int) Nonzero if game can't continue

***************************************************************************/

int      post_init (void)
{
   int x, y;

   x = MIN (w, h);
   scale = fdiv (I2F (x), I2F (SIM_SIZE));
   lowx = SCALE_I2F (5 + SHIP_SIZE / 2);
   highx = SCALE_I2F (SIM_SIZE - 5 - SHIP_SIZE / 2);
   deltax = SCALE_F2F (fdiv (fdiv (I2F (SIM_SIZE), I2F (4)), I2F (TICKS)));

   back = CB (x, x);

   y = SCALE_I2I (SHIP_SIZE);
   
   ship = CB (y, y / 4);
   sub = CB (y, y / 4);
   
   y = SCALE_I2I (MINE_SIZE);
   
   mine = CB (y, y);
   load = CB (y, y);
   table = CB (text_length (font, " x x x x x 00000 x x x x x "), text_height (font));
   
   if (!back || !ship || !sub || !mine || !load || !table)
   {
      SHOW_OUT ("not_enough_mem");
      return 1;
   }

   /* Let's draw something */
   clear (ship);
   clear (sub);
   clear (mine);
   clear_to_color (load, WHITE);

   line (ship, 0, 0, BORDER, ship->h- 1, WHITE);
   line (ship, ship->w-1, 0, ship->w- 1 - BORDER, ship->h-1, WHITE);
   floodfill (ship, 1, 0, WHITE);

   y = sub->h / 2;
   for (x = y; x < sub->w - y; x++)
      circlefill (sub, x, y, y - 1, WHITE);

   line (mine, 0, 0, mine->w-1, mine->h-1, WHITE);
   line (mine, mine->w-1, 0, 0, mine->h-1, WHITE);

   return 0;
}


